Index: pci_cfgreg.c
===================================================================
RCS file: /cache/ncvs/src/sys/i386/isa/Attic/pci_cfgreg.c,v
retrieving revision 1.1.2.1
diff -u -r1.1.2.1 pci_cfgreg.c
--- pci_cfgreg.c	2001/07/28 05:55:07	1.1.2.1
+++ pci_cfgreg.c	2001/08/17 07:11:44
@@ -54,7 +54,10 @@
 static int cfgmech;
 static int devmax;
 static int usebios;
+static int ignore_pcibios;
 
+TUNABLE_INT("hw.pci.ignore_pcibios", &ignore_pcibios);
+
 static int	pci_cfgintr_unique(struct PIR_entry *pe, int pin);
 static int	pci_cfgintr_linked(struct PIR_entry *pe, int pin);
 static int	pci_cfgintr_search(struct PIR_entry *pe, int bus, int device, int matchpin, int pin);
@@ -195,6 +198,19 @@
 }
 
 
+static u_int16_t
+pcibios_get_version(void)
+{
+    struct bios_regs args;
+
+    args.eax = PCIBIOS_BIOS_PRESENT;
+    if (bios32(&args, PCIbios.ventry, GSEL(GCODE_SEL, SEL_KPL)))
+	return (0xffff);
+    if (args.edx != 0x20494350)
+	return (0xffff);
+    return (args.ebx & 0xffff);
+}
+
 /*
  * Route a PCI interrupt
  *
@@ -208,7 +224,15 @@
     struct PIR_entry	*pe;
     int			i, irq;
     struct bios_regs	args;
+    u_int16_t		v;
     
+    v = pcibios_get_version();
+    if (v < 0x0210) {
+	printf(
+	  "pci_cfgintr: BIOS %x.%02x doesn't support interrupt routing\n",
+	  (v & 0xff00) >> 8, v & 0xff);
+	return (255);
+    }
     if ((bus < 0) || (bus > 255) || (device < 0) || (device > 255) ||
       (pin < 1) || (pin > 4))
 	return(255);
@@ -236,12 +260,14 @@
 	args.eax = PCIBIOS_ROUTE_INTERRUPT;
 	args.ebx = (bus << 8) | (device << 3);
 	args.ecx = (irq << 8) | (0xa + pin - 1);	/* pin value is 0xa - 0xd */
-	bios32(&args, PCIbios.ventry, GSEL(GCODE_SEL, SEL_KPL));
-
-	/*
-	 * XXX if it fails, we should try to smack the router hardware directly
-	 */
-
+	if (bios32(&args, PCIbios.ventry, GSEL(GCODE_SEL, SEL_KPL))) {
+		/*
+		 * XXX if it fails, we should try to smack the router
+		 * hardware directly
+		 */
+	    printf("pci_cfgintr: ROUTE_INTERRUPT failed\n");
+	    return (255);
+	}
 	printf("pci_cfgintr: %d:%d INT%c routed to irq %d\n", 
 	       bus, device, 'A' + pin - 1, irq);
 	return(irq);
@@ -454,8 +480,19 @@
 static int
 pcibios_cfgopen(void)
 {
+    u_int16_t		v;
+    
+    if (PCIbios.entry) {
+	if (ignore_pcibios) {
+	    printf("PCI code ignoring PCIbios entry\n");
+	} else {
+	    v = pcibios_get_version();
+	    printf("pci_cfgintr: BIOS version %x.%02x\n", (v & 0xff00) >> 8,
+		v & 0xff);
+	}
+    }
     /* check for a found entrypoint */
-    return(PCIbios.entry != 0);
+    return(PCIbios.entry != 0 && ignore_pcibios == 0);
 }
 
 /* 
